#pragma once

#include <stddef.h>
#include <stdint.h>

#include "immersitech_general.h"

#if HAVE_VISIBILITY
#define IMMERSITECH_API __attribute__((__visibility__("default")))
#elif defined _WIN32
#define IMMERSITECH_API __declspec(dllexport)
#else
#define IMMERSITECH_API
#endif

/// @brief The different operating modes of AI Noise Cancellation
typedef enum {
    IMM_CV_ANC_START,   ///< Start value
    IMM_CV_ANC_OFF,     ///< AI Noise Cancellation disabled
    IMM_CV_ANC_HIGH,    ///< AI Noise Cancellation at full power mode
    IMM_CV_ANC_LOW,     ///< AI Noise Cancellation at low power mode
    IMM_CV_ANC_END      ///< End value
} imm_cv_anc_mode;

/// @brief The different operating modes of Voice Activity Detection
typedef enum {
    IMM_CV_VAD_START,                       ///< Start value
    IMM_CV_VAD_FREQ_ANALYSIS,               ///< VAD analyzing only frequency content for detection
    IMM_CV_VAD_FREQ_AND_AMPLITUDE_ANALYSIS, ///< VAD analyzing frequency content and amplitude for detection
    IMM_CV_VAD_END                          ///< End value
} imm_cv_vad_mode;

/// @brief The configurations of the ClearVoice library that can be changed after initialization
typedef enum {
    IMM_CV_MUT_CONF_START,                          ///< Start value
    IMM_CV_MUT_CONF_ANC_MODE,                       ///< The operating mode of AI Noise Cancellation
    IMM_CV_MUT_CONF_AEQ_ENABLED,                    ///< Whether Automatic Equalization is On or Off
    IMM_CV_MUT_CONF_AGC_ENABLED,                    ///< Whether Automatic Gain Control is On or Off
    IMM_CV_MUT_CONF_ANC_MIX,                        ///< The wet/dry mix [0-100] of the AI Noise Cancellation. 0 is only noisy audio, 100 is only noise cancelled audio.
    IMM_CV_MUT_CONF_BSD_LOW_VAD_THRESHOLD,          ///< The VAD Score threshold considered speech when calculating Binary Speech Detection (BSD)
    IMM_CV_MUT_CONF_BSD_INACTIVE_SPEECH_COOLDOWN,   ///< The time (ms) the VAD Score must be under bsd_low_vad_threshold in order to BSD to return False
    IMM_CV_MUT_CONF_VAD_MODE,                       ///< The operating mode of the Voice Activity Detector
    IMM_CV_MUT_CONF_END                             ///< End value
} imm_cv_mutable_config;

/// @brief A structure containing all necessary configuration options to initialize ClearVoice.
///
/// @param input_sample_rate The sample rate of the audio input.
/// @param output_sample_rate The desired sample rate of the audio output.
/// @param anc_mode The operating mode of AI Noise Cancellation
/// @param aeq_enabled Whether Automatic Equalization is On or Off
/// @param agc_enabled Whether Automatic Gain Control is On or Off
/// @param anc_mix The wet/dry mix [0-100] of the AI Noise Cancellation. 0 is only noisy audio, 100 is only noise cancelled audio.
/// @param bsd_low_vad_threshold The VAD Score threshold considered speech when calculating Binary Speech Detection (BSD).
/// @param bsd_inactive_speech_cooldown The time in ms the VAD Score must be under bsd_low_vad_threshold in order to BSD to return False.
/// @param vad_mode The operating mode of the Voice Activity Detector
/// @param multistream_mode_enabled Whether ClearVoice is operating in Multistream Mode or not. In Multistream Mode, you must add individual audio streams with imm_cv_add_stream(), and process them using imm_cv_process_stream().
typedef struct {
    int input_sample_rate;
    int output_sample_rate;
    imm_cv_anc_mode anc_mode;
    int aeq_enabled;
    int agc_enabled;
    int anc_mix;
    int bsd_low_vad_threshold;
    int bsd_inactive_speech_cooldown;
    imm_cv_vad_mode vad_mode;
    int multistream_mode_enabled;
} imm_cv_config;

/// @brief A structure to return all extra data from the process function.
///
/// @param vad_score The likelihood [0-1] of speech present in the output block.
/// @param speech_active True if speech is present in the output frame, false if not. Configurable with the Binary Speech Detection (BSD) parameters.
typedef struct {
    float vad_score;
    bool speech_active;
} imm_cv_output_metadata;

typedef void* imm_cv_handle;

#ifdef __cplusplus
extern "C" {
#endif

/// @brief Return a default configuration object for initialization
IMMERSITECH_API imm_cv_config imm_cv_get_default_config();

/// @brief Initialize an instance of the Immersitech ClearVoice Library
///
/// @param[in] license_file_path The path to your Immersitech license file
/// @param[in] config An ::imm_cv_config struct
/// @param[out] error_code An ::imm_error_code representing if the call was a success or not
///
/// @return An ::imm_cv_handle representing the initilized Immersitech Library. 
IMMERSITECH_API imm_cv_handle imm_cv_init_from_file(const char* license_file_path, imm_cv_config config, imm_error_code* error_code);

/// @brief Initialize an instance of the Immersitech ClearVoice Library
///
/// @param[in] license_file_data Immersitech license file data
/// @param[in] license_file_data_size The size of the Immersitech license file data
/// @param[in] config An ::imm_cv_config struct
/// @param[out] error_code An ::imm_error_code representing if the call was a success or not
///
/// @return An ::imm_cv_handle representing the initilized Immersitech Library.
IMMERSITECH_API imm_cv_handle imm_cv_init_from_memory(uint8_t* license_file_data, size_t license_file_data_size, imm_cv_config config, imm_error_code* error_code);

/// @brief Destroy an instance of the Immersitech ClearVoice Library
IMMERSITECH_API imm_error_code imm_cv_destroy(imm_cv_handle handle);

/// @brief Set a configuration value
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] config An ::imm_cv_mutable_config representing the configuration you'd like to change
/// @param[in] val The value you'd like to set the configuration to
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_set_config(imm_cv_handle handle, imm_cv_mutable_config config, int val);

/// @brief Get a configuration value
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] config An ::imm_cv_mutable_config representing the configuration you'd like to get
/// @param[out] val A pointer to the value you'd like to get
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_get_config(imm_cv_handle handle, imm_cv_mutable_config config, int* val);

/// @brief Process audio through the Immersitech ClearVoice Library
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] input A pointer to your input audio data
/// @param[out] output A pointer to your output audio data
/// @param[out] output_metadata A pointer to an ::imm_cv_output_metadata struct, which will hold metadata about the output audio data
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_process(imm_cv_handle handle, float* input, float* output, imm_cv_output_metadata* output_metadata);

/// @brief Add a stream for audio processing.
///
/// This function will only work if multistream_mode was set to true when ClearVoice was initialized.
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[out] id An ID used to identify this stream in the future.
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_add_stream(imm_cv_handle handle, int* id);

/// @brief Destroy a previously created stream.
///
/// This function will only work if multistream_mode was set to true when ClearVoice was initialized.
/// All new streams will take on the configuration settings used to initialize the library until changed
/// with imm_cv_set_stream_config().
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] id An ID previously returned from ::imm_cv_add_stream() 
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_destroy_stream(imm_cv_handle handle, int id);

/// @brief Process audio through the Immersitech ClearVoice Library
///
/// This function will only work if multistream_mode was set to true when ClearVoice was initialized.
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] id An ID representing which stream this audio data belongs to
/// @param[in] input A pointer to your input audio data
/// @param[out] output A pointer to your output audio data
/// @param[out] output_metadata A pointer to an ::imm_cv_output_metadata struct, which will hold metadata about the output audio data
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_process_stream(imm_cv_handle handle, int id, float* input, float* output, imm_cv_output_metadata* output_metadata);

/// @brief Set a configuration value
///
/// This function will only work if multistream_mode was set to true when ClearVoice was initialized.
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] id An ID representing which stream this configuration refers to
/// @param[in] config An ::imm_cv_mutable_config representing the configuration you'd like to change
/// @param[in] val The value you'd like to set the configuration to
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_set_stream_config(imm_cv_handle handle, int id, imm_cv_mutable_config config, int val);

/// @brief Get a configuration value
///
/// This function will only work if multistream_mode was set to true when ClearVoice was initialized.
///
/// @param[in] handle An ::imm_cv_handle to a ClearVoice instance
/// @param[in] id An ID representing which stream this configuration refers to
/// @param[in] config An ::imm_cv_mutable_config representing the configuration you'd like to get
/// @param[out] val A pointer to the value you'd like to get
///
/// @return An ::imm_error_code representing if the call was a success or not
IMMERSITECH_API imm_error_code imm_cv_get_stream_config(imm_cv_handle handle, int id, imm_cv_mutable_config config, int* val);

#ifdef __cplusplus
}
#endif